@@ -11,7 +11,13 @@ from django.conf import settings
 from django.contrib.admin import widgets as admin_widgets
 from django.core.urlresolvers import reverse
 from django.forms.widgets import flatatt
-from django.forms.util import smart_unicode
+try:
+    from django.utils.encoding import smart_text as smart_unicode
+except ImportError:
+    try:
+        from django.utils.encoding import smart_unicode
+    except ImportError:
+        from django.forms.util import smart_unicode
 from django.utils.html import escape
 from django.utils import simplejson
 from django.utils.datastructures import SortedDict
@@ -38,8 +44,10 @@ class TinyMCE(forms.Textarea):
     parameter.
     """
 
-    def __init__(self, content_language=None, attrs=None, mce_attrs={}):
+    def __init__(self, content_language=None, attrs=None, mce_attrs=None):
         super(TinyMCE, self).__init__(attrs)
+        if mce_attrs is None:
+            mce_attrs = {}
         self.mce_attrs = mce_attrs
         if content_language is None:
             content_language = mce_attrs.get('language', None)
@@ -55,25 +63,64 @@ class TinyMCE(forms.Textarea):
         mce_config = tinymce.settings.DEFAULT_CONFIG.copy()
         mce_config.update(get_language_config(self.content_language))
         if tinymce.settings.USE_FILEBROWSER:
-            mce_config['file_browser_callback'] = "DjangoFileBrowser"
+            mce_config['file_browser_callback'] = "djangoFileBrowser"
         mce_config.update(self.mce_attrs)
-        mce_config['mode'] = 'exact'
-        mce_config['elements'] = final_attrs['id']
+        if not 'mode' in mce_config:
+            mce_config['mode'] = 'exact'
+        if mce_config['mode'] == 'exact':
+            mce_config['elements'] = final_attrs['id']
         mce_config['strict_loading_mode'] = 1
+        
+        # Fix for js functions
+        js_functions = {}
+        for k in ('paste_preprocess','paste_postprocess'):
+            if k in mce_config:
+               js_functions[k] = mce_config[k]
+               del mce_config[k]
         mce_json = simplejson.dumps(mce_config)
 
+        pos = final_attrs['id'].find('__prefix__')
+        if pos != -1:
+            mce_json = mce_json.replace(u'"%s"' % final_attrs['id'], u'elements')
+
+        for k in js_functions:
+            index = mce_json.rfind('}')
+            mce_json = mce_json[:index]+', '+k+':'+js_functions[k].strip()+mce_json[index:]
+            
+
         html = [u'<textarea%s>%s</textarea>' % (flatatt(final_attrs), escape(value))]
         if tinymce.settings.USE_COMPRESSOR:
             compressor_config = {
-                'plugins': mce_config['plugins'],
-                'themes': mce_config['theme'],
-                'languages': mce_config['language'],
+                'plugins': mce_config.get('plugins', ''),
+                'themes': mce_config.get('theme', 'advanced'),
+                'languages': mce_config.get('language', ''),
                 'diskcache': True,
                 'debug': False,
             }
             compressor_json = simplejson.dumps(compressor_config)
-            html.append(u'<script type="text/javascript">tinyMCE_GZ.init(%s)</script>' % compressor_json)
-        html.append(u'<script type="text/javascript">tinyMCE.init(%s)</script>' % mce_json)
+            
+        if pos != -1:
+            html.append(u'''<script type="text/javascript">
+setTimeout(function () {
+    var id = '%s';
+    
+    if (typeof(window._tinymce_inited) == 'undefined') {
+        window._tinymce_inited = [];
+    }
+    
+    if (typeof(window._tinymce_inited[id]) == 'undefined') {
+        window._tinymce_inited[id] = true;
+    } else {
+        var elements = id.replace(/__prefix__/, parseInt(document.getElementById('%sTOTAL_FORMS').value) - 1);
+        console.log(elements);
+        if (document.getElementById(elements)) {
+            tinymce.init(%s);
+        }
+    }
+}, 0);
+</script>''' % (final_attrs['id'], final_attrs['id'][0:pos], mce_json))
+        else:
+            html.append(u'<script type="text/javascript">tinyMCE.init(%s)</script>' % mce_json)
 
         return mark_safe(u'\n'.join(html))
 
